// -*- C++ -*-

#ifndef PACKAGE_H
#define PACKAGE_H

#include "board.h"
#include "util.h"
#include <iosfwd>
#include <map>
#include <set>

using std::set;

class package_type;
class robot_type;

typedef set<package_type *> package_vector;

#include "robot.h"

using std::ostream;
using std::map;
using std::multimap;

// All packages indexed by their location.
typedef multimap<loc_type, package_type *> packages_by_loc_type;
extern packages_by_loc_type packages_by_loc;
typedef map<int, package_type *> package_map;
extern package_map packages;

class package_type {
public:
    static stat weight_stat;
    
    int id;
    loc_type *destination;
private:
    int *_weight;
    
    // Either location or carrier is NULL, but never both.
    loc_type *_location;
    robot_type *_carrier;
    bool delivered;
public:
    
    package_type(int iid, robot_type *carrier);
    package_type(int iid, loc_type const &idest, int w, loc_type const &iloc);
    
    ~package_type() {
        delete destination;
        delete _weight;
        delete _location;
    }
    
    void print(ostream &out);

    loc_type const *location() const {
        return _location;
    }

    robot_type const *carrier() const {
        return _carrier;
    }

    int weight() const {
        return _weight ? *_weight : (int) weight_stat.average();
    }

    int *real_weight() const {
        return _weight;
    }
    
    void set_weight(int w) {
        if (_weight == 0)
            weight_stat.add_double(w);
        _weight = new int(w);
    }

    void set_destination(loc_type const &dst) {
        if (destination == 0)
            destination = new loc_type(dst);
    }
    
    void set_location(loc_type const &loc);
    void set_carrier(robot_type *robot);
    float get_value() const;
    float get_value(loc_type const & from, int penalty) const;
    
    void set_delivered() {delivered = true;}
    bool get_delivered() const {return delivered;}
    
private:
    package_type(const package_type &rhs);
    package_type const &operator=(package_type const &rhs);
};

ostream &operator<<(ostream &out, package_type const &rhs);

//package_type *get_package(int id);
package_type *get_package(int id, robot_type *carrier);
package_type *get_package(int id, loc_type destination, int weight,
                          loc_type location);

void print_packages(ostream &out);

#endif // PACKAGE_H
