#include "robot.h"
#include "util.h"
#include <ostream>
#include <stdexcept>
#include <utility>

using namespace std;

robot_type *self;

robot_map robots;

static int robot_capacity;
static int robot_money;

robot_type *
get_robot(int id) {
    robot_type *result
        = (*robots.insert(make_pair(id, (robot_type *) 0)).first).second;
    if (result == 0) {
        result = robots[id] = new robot_type(id, robot_capacity, robot_money);
    } 
    return result;
}

robot_type *
read_robot(int fd) {
    string config;
    if (!readline(fd, config)) {
        return 0;
    }

    int id;
    int rc = sscanf(config.c_str(), "%d %d %d", &id, &robot_capacity, &robot_money);
    if (rc != 3) {
        throw std::runtime_error("bad robot configuration");
    }

    self = get_robot(id);
    return self;
}

void
robot_type::print(ostream &out) {
    out << *this << '\n';
}

void
robot_type::before_results() {
    _moves = 0;
    _picks = 0;
    _drops = 0;
    _dead = true;
}

void
robot_type::not_dead() {
    _dead = false;
}

bool
robot_type::is_pushed() const {
    return (_moves > 1
            || (_moves > 0 && _picks > 0)
            || (_moves > 0 && _drops > 0)
            || (_picks > 0 && _drops > 0));
}

void
robot_type::move(dir_type dir) {
    ++_moves;
    location = location.move(dir);
}

void
robot_type::add(package_type *package) {
    assert(package->carrier() == this);
    ++_picks;
    if (packages.insert(package).second)
        _capacity -= package->weight();
}

void
robot_type::remove(package_type *package) {
    assert(package->carrier() == this);
    ++_drops;
    if (packages.erase(package) > 0)
        _capacity += package->weight();
}

ostream &
operator<<(ostream &out, robot_type const &rhs) {
    return out << '#' << rhs.id << ' ' << robot_capacity << ' '
               << robot_money << rhs.location;
}

void
print_robots(ostream &out) {
    out << "Robots:\n";
    for (robot_map::iterator it = robots.begin(); it != robots.end(); ++it) {
        (*it).second->print(out);
    }
}
