// -*- C++ -*-

#ifndef ROBOT_H
#define ROBOT_H

#include "board.h"
#include "package.h"
#include <iosfwd>
#include <map>

using std::ostream;
using std::map;

class robot_type {
private:
    int _init_capacity;
    int _init_money;
    int _capacity;
    int _money;
    int _moves;
    int _picks;
    int _drops;
    bool _dead;
public:
    int id;
    loc_type location;
    package_vector packages;

    robot_type(int iid, int icap, int imoney)
        : _init_capacity(icap), _init_money(imoney), _capacity(icap),
          _money(imoney),
          _moves(0), _picks(0), _drops(0), _dead(true),
          id(iid), location(), packages()
        {}

    int max_capacity() const {
        return _init_capacity;
    }
    int capacity() const { return _capacity; }
    int encumbrance() const { return _init_capacity - _capacity; }

    void used_money(int amount) {
        _money -= amount;
    }

    int money() const {
        return _money;
    }
    
    void move(dir_type dir);
    void print(ostream &out);

    void before_results();
    void not_dead();
    bool is_pushed() const;
    bool is_dead() const { return _dead; }
private:
    friend class package_type;
    
    void add(package_type *package);
    void remove(package_type *package);
    
    robot_type(const robot_type &rhs);
    robot_type const &operator=(robot_type const &rhs);
};

ostream &operator<<(ostream &out, robot_type const &rhs);

extern robot_type *self;

typedef map<int, robot_type *> robot_map;
extern robot_map robots;

static inline size_t live_robot_count() {
    size_t result = 0;
    for (robot_map::iterator it = robots.begin();
         it != robots.end();
         ++it)
    {
        if (!(*it).second->is_dead())
            ++result;
    }
    return result;
}

robot_type *get_robot(int id);
robot_type *read_robot(int fd);

void print_robots(ostream &out);

#endif // ROBOT_H
