#include "util.h"
#include "timer.h"
#include <cmath>
#include <stdexcept>
#include <unistd.h>
#include <errno.h>
#include <iostream>

using namespace std;

const int MAXLINE = 1024;

static ssize_t
my_read(int fd, char *ptr) {
    static int read_cnt = 0;
    static char *read_ptr;
    static char read_buf[MAXLINE];

    if (read_cnt <= 0) {
    again:
        if ( (read_cnt = read(fd, read_buf, sizeof(read_buf))) < 0) {
            if (errno == EINTR)
                goto again;
            return -1;
        } else if (read_cnt == 0)
            return 0;
        read_ptr = read_buf;
    }
    read_cnt--;
    *ptr = *read_ptr++;
    return 1;
}

bool
readline(int fd, string &s) {
    ssize_t rc;
    char c;
    
    s.clear();
    while (1) {
        if ((rc = my_read(fd, &c)) == 1) {
            if (c == '\n')
                break;          // newline is not stored
            s += c;
        } else if (rc == 0) {
            if (s.size() == 0) {
                return false;   // EOF, no data read
            } else {
                break;
            }
        } else {
            if (errno == EINTR)
                continue;
            else if (errno == EPIPE)
                return false;
            else {
                cerr << "my_read: " << strerror(errno) << endl;
                throw std::runtime_error("error reading socket");
            }
        }
    }

#ifndef NDEBUG
    cerr << "Read '" << s.substr(0, 250) << "'\n";
#endif
    
    return true;
}

void
sendline(int fd, string const &s) {
    string s2(s);
    s2 += '\n';
#ifndef NDEBUG
    cerr << "Send '" << s.substr(0, 50) << "'\n";
#endif
    int rc = write(fd, s2.data(), s2.size());
    if (rc != s2.size()) {
        throw std::runtime_error("error writing packet");
    }
}

stat::stat()
    : _count(0), _count_all(0),
      _sum(0.0), _sum_of_squares(0.0),
      _min(std::numeric_limits<double>::max()),
      _max(std::numeric_limits<double>::min())
{
}
    
stat::~stat() {
}

void
stat::add_null() {
    ++_count_all;
}

void
stat::add_double(double const &value) {
    ++_count;
    ++_count_all;
    _sum += value;
    _sum_of_squares += value * value;
    _min = std::min(_min, value);
    _max = std::max(_max, value);
}
    
double
stat::variance() const {
    return (sum_of_squares() - sum() * sum() / count()) / (count() - 1);
}

double
stat::stdev() const {
    return sqrt(variance());
}

ostream &
operator<<(ostream &out, stat const &rhs) {
    return out << "count: " << rhs.count()
               << ", sum: " << rhs.sum()
               << ", min: " << rhs.min()
               << ", max: " << rhs.max()
               << ", avg: " << rhs.average()
               << ", stdev: " << rhs.stdev();
}
