#!/usr/bin/perl

$MAX_X = 1000;
$MAX_Y = 1000;
$MAX_PACKAGES = 10000;
$MAX_WEIGHT = 1000000000;
$MAX_CAPACITY = 1000000000;
$MAX_FUEL = 1000000000;
$SIMULATOR = "/home/martin/Simulator";
$CLIENT = "../source/client";

@home_bases = ();
$total_bases = 0;

sub getRandomTile {
    my $x = shift;
    my $y = shift;

    my $randval = rand(1);

    if($randval < .85) {
        return ".";
    }
    if($randval < .93) {
        return "#";
    }
    if($randval < .995) {
        return "~";
    }
    my @pair = ($x+1,$y+1);
    push(@home_bases, \@pair);
    #print "base at $x, $y\n";
    $total_bases++;
    return "@";
}

use integer;

my $fieldx = rand($MAX_X - 1) + 1;
my $fieldy = rand($MAX_Y - 1) + 1;
my $capacity = rand($MAX_CAPACITY - 1) + 1;
my $fuel = 2000;

my @field;

print "* Generating field of size $fieldx by $fieldy\n";

# Gen field
open(OUTPUT, "> gen.map") || die "Can't open output map";
for($cury = 0; $cury < $fieldy; $cury++) {
    for($curx = 0; $curx < $fieldx; $curx++) {
        my $tile = getRandomTile($curx,$cury);
        $field[$curx][$cury] = $tile;
        print OUTPUT "$tile";
    }
    print OUTPUT "\n";
}
close(OUTPUT);

# Gen packs
open(OUTPUT, "> gen.packs") || die "Can't open output packs";
print OUTPUT "[\n";
my $pack_number = 0;
my $base = 0;
my $total_packs = rand($fieldx * $fieldy);
$total_packs = $MAX_PACKAGES if($total_packs > $MAX_PACKAGES);

foreach $pair (@home_bases) {
    my ($x, $y) = @$pair;
    # print "Homebase at $x, $y\n";
    print OUTPUT "(($x,$y),\n";
    print OUTPUT "[\n";
    my $numpacks = rand($total_packs / $total_bases) + 1;
    $numpacks = 0 if($pack_number + $numpacks > $MAX_PACKAGES);

    for ($i=0; $i<$numpacks; $i++) {
        my $weight = rand($capacity) + rand(100); # sometimes too heavy
        my $destx = rand($fieldx - 1) + 1;
        my $desty = rand($fieldy - 1) + 1;
        printf(OUTPUT "Mk {uid=%d, dest=(%d,%d), weight=%d}", 
               $pack_number, $destx, $desty, $weight);
        print OUTPUT "," if($i < $numpacks - 1);
        print OUTPUT "\n";
        $pack_number++;
    }
    print OUTPUT "])";
    print OUTPUT "," if($base < $#home_bases);
    print OUTPUT "\n";
    $base++;
}
print OUTPUT "]\n";
close(OUTPUT);

my $cmd = sprintf("$SIMULATOR -p 3001 -n 1 -m gen.map -k gen.packs -f %d -c %d\n", 
    $fuel, $capacity);
print "$cmd\n";

# Start simulator
my $pid = fork();
if($pid == 0) {
    `$cmd`;
} else {
    my $simpid = $pid;
    print "Simulator PID = $simpid\n";
    sleep(15);

# Start client
    print "Starting client\n";
    my $result = `$CLIENT localhost 3001 > client.out`;
    print "Done.\n";

    if ($?) {
        my $name = `date +%d%m%Y-%k%M%S`;
        chomp $name;
        `mv gen.map $name.map`;
        `mv gen.packs $name.packs`;
        print "CLIENT FAILED!\n";
    }

    my @pids = (++$simpid);
    print `kill -9 $simpid` . "\n";
    #wait();
}
